"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bignumber_js_1 = require("bignumber.js");
var DICTIONARY = 'ABCDEFGHJKLMNOPQRSTUVWXYZabcdefhijkmnopqrstuvwxyz23456789';
var DICTIONARY_LENGTH = DICTIONARY.length;
var SHARECODE_PATTERN = /CSGO(-?[\w]{5}){5}$/;
/**
 * Convert a byte array into a hexadecimal string
 */
function bytesToHex(bytes) {
    return Array.from(bytes, function (byte) {
        return ('0' + (byte & 0xff).toString(16)).slice(-2);
    }).join('');
}
/**
 * Convert a BigNumber into a byte array.
 */
function bigNumberToByteArray(big) {
    var str = big.toString(16).padStart(36, '0');
    var bytes = [];
    for (var i = 0; i < str.length; i += 2) {
        bytes.push(parseInt(str.slice(i, i + 2), 16));
    }
    return bytes;
}
/**
 * Convert a 64 bit 2 complement integer into a byte array (big endian byte representation)
 */
function longToBytesBE(high, low) {
    return [
        (high >>> 24) & 0xff,
        (high >>> 16) & 0xff,
        (high >>> 8) & 0xff,
        high & 0xff,
        (low >>> 24) & 0xff,
        (low >>> 16) & 0xff,
        (low >>> 8) & 0xff,
        low & 0xff,
    ];
}
/**
 * Convert an int into a byte array (low bits only)
 */
function int16ToBytes(number) {
    return [(number & 0x0000ff00) >> 8, number & 0x000000ff];
}
/**
 * Convert a byte array into an int32
 */
function bytesToInt32(bytes) {
    var number = 0;
    for (var i = 0; i < bytes.length; i++) {
        number += bytes[i];
        if (i < bytes.length - 1) {
            number = number << 8;
        }
    }
    return number;
}
/**
 * Encode a share code from its ShareCode object type and return its string representation.
 * Required fields should come from a CDataGCCStrike15_v2_MatchInfo protobuf message.
 * https://github.com/SteamRE/SteamKit/blob/master/Resources/Protobufs/csgo/cstrike15_gcmessages.proto#L785
 */
var encode = function (matchId, reservationId, tvPort) {
    var matchBytes = longToBytesBE(matchId.high, matchId.low).reverse();
    var reservationBytes = longToBytesBE(reservationId.high, reservationId.low).reverse();
    var tvBytes = int16ToBytes(tvPort).reverse();
    var bytes = Array.prototype.concat(matchBytes, reservationBytes, tvBytes);
    var bytesHex = bytesToHex(bytes);
    var total = new bignumber_js_1.default(bytesHex, 16);
    var c = '';
    var rem = new bignumber_js_1.default(0);
    for (var i = 0; i < 25; i++) {
        rem = total.mod(DICTIONARY_LENGTH);
        c += DICTIONARY[rem.integerValue(bignumber_js_1.default.ROUND_FLOOR).toNumber()];
        total = total.div(DICTIONARY_LENGTH);
    }
    return "CSGO-" + c.substr(0, 5) + "-" + c.substr(5, 5) + "-" + c.substr(10, 5) + "-" + c.substr(15, 5) + "-" + c.substr(20, 5);
};
exports.encode = encode;
/**
 * Decode a CSGO share code from its string and return it as a ShareCode object type.
 * Share code format excepted: CSGO-xxxxx-xxxxx-xxxxx-xxxxx-xxxxx
 */
var decode = function (shareCode) {
    if (!shareCode.match(SHARECODE_PATTERN)) {
        throw new Error('Invalid share code');
    }
    shareCode = shareCode.replace(/CSGO|-/g, '');
    var chars = Array.from(shareCode).reverse();
    var big = new bignumber_js_1.default(0);
    for (var i = 0; i < chars.length; i++) {
        big = big.multipliedBy(DICTIONARY_LENGTH).plus(DICTIONARY.indexOf(chars[i]));
    }
    var bytes = bigNumberToByteArray(big);
    return {
        matchId: {
            low: bytesToInt32(bytes.slice(0, 4).reverse()),
            high: bytesToInt32(bytes.slice(4, 8).reverse()),
        },
        reservationId: {
            low: bytesToInt32(bytes.slice(8, 12).reverse()),
            high: bytesToInt32(bytes.slice(12, 16).reverse()),
        },
        tvPort: bytesToInt32(bytes.slice(16, 18).reverse()),
    };
};
exports.decode = decode;
//# sourceMappingURL=index.js.map